//=========================================================
// LPC1114 Project
//=========================================================
// File Name : main.c
// Function  : Main Routine
//---------------------------------------------------------
// Rev.01 2011.02.26 Munetomo Maruyama
//---------------------------------------------------------
// Copyright (C) 2010-2011 Munetomo Maruyama
//=========================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include "array_com.h"
#include "color_led.h"
#include "oled.h"
#include "mems.h"
#include "systick.h"
#include "uart.h"

//===============================
// Mary Rover Controller
//===============================
//-----------------------
// Main Routine
//-----------------------
int main(void)
{
    uint32_t tick_curr;
    uint32_t tick_intv;
    uint32_t tick_prev = 0;
    //
    // Initialization
    //
    Init_SysTick();
    Init_Array_COM();
    Array_COM_ID_Assignment();
    Init_Color_LED();
    //
    // I am XB.
    //
    if (Array_COM_Get_ID() == 0)
    {
        //
        // Initialize Hardware
        //
        UARTInit(9600);
        //
        // Forever Loop
        //
        while(1)
        {
            uint32_t port;
            uint32_t got;
            uint32_t count;
            uint32_t rx_data;
            int8_t mems_x;
            int8_t mems_y;
            int8_t mems_z;
            //
            // Receive MEMS Data from Adjacent
            //
            got = 0;
            for (port = 0; port < 4; port++)
            {
                if (Array_COM_Port_Open(port))
                {
                    if ((count = Array_COM_Rx_Data32(port, &rx_data, 0)) > 0)
                    {
                        mems_x = (int8_t) ((rx_data >> 16) & 0x0ff);
                        mems_y = (int8_t) ((rx_data >>  8) & 0x0ff);
                        mems_z = (int8_t) ((rx_data >>  0) & 0x0ff);
                        got = 1;
                    }
                }
            }
            //
            // Send the MEMS Data from XBee
            //
            if (got)
            {
                UARTSend_Byte(0x80); // start mark
                UARTSend_Byte((uint8_t) mems_x);
                UARTSend_Byte((uint8_t) mems_y);
                UARTSend_Byte((uint8_t) mems_z);
            }
        }
    }
    //
    // I am OB.
    //
    else
    {
        //
        // Initialize Hardware
        //
        Init_OLED();
        Init_MEMS();
        //
        OLED_Clear_Screen(OLED_BLK); // Clear Screen
        OLED_printf_Font(OLED_FONT_MEDIUM);
        OLED_printf_Position(0, 0);
        OLED_printf_Color(OLED_YEL, OLED_BLK);
        OLED_printf("MARY");
        OLED_printf_Font(OLED_FONT_SMALL);
        OLED_printf_Position(0, 2);
        OLED_printf_Color(OLED_GRN, OLED_BLK);
        OLED_printf("Rover Control");
        //
        // Forever Loop
        //
        while(1)
        {
            uint32_t port;
            int8_t mems_x;
            int8_t mems_y;
            int8_t mems_z;
            uint32_t tx_data;
            //
            // Operation Cycle
            //
            while(1)
            {
                tick_curr = Get_Ticks();
                tick_intv = tick_curr - tick_prev;
                if (tick_intv > 20) break; // 200ms
            }
            tick_prev = tick_curr;
            //
            // Get MEMS Data
            //
            mems_x = MEMS_Get_X();
            mems_y = MEMS_Get_Y();
            mems_z = MEMS_Get_Z();
            tx_data = ((mems_x & 0xff) << 16) | ((mems_y & 0xff) << 8) | ((mems_z & 0xff) << 0);
            //
            // Send MEMS Data to Adjacent
            //
            for (port = 0; port < 4; port++)
            {
                if (Array_COM_Port_Open(port))
                {
                    Array_COM_Tx_Data32(port, tx_data);
                }
            }
            //
            // Display MEMS Data on OLED
            //
            OLED_printf_Font(OLED_FONT_MEDIUM);
            OLED_printf_Position(0, 3);
            OLED_printf_Color(OLED_RED, OLED_BLK);
            OLED_printf("X=%3d\n", (int32_t) mems_x);
            OLED_printf_Position(0, 4);
            OLED_printf_Color(OLED_GRN, OLED_BLK);
            OLED_printf("Y=%3d\n", (int32_t) mems_y);
            OLED_printf_Position(0, 5);
            OLED_printf_Color(OLED_BLU, OLED_BLK);
            OLED_printf("Z=%3d\n", (int32_t) mems_z);
        }

    }

    return 0;
}

//=========================================================
// End of Program
//=========================================================
